@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/aticonsole.c,v 1.22 2003/11/13 18:42:47 tsi Exp $ */
 /*
  * Copyright 1997 through 2004 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
@@ -30,9 +29,7 @@
 #endif
 
 #include "ati.h"
-#include "atiadapter.h"
 #include "aticonsole.h"
-#include "aticrtc.h"
 #include "atii2c.h"
 #include "atilock.h"
 #include "atimach64.h"
@@ -47,12 +44,6 @@
 #include "atidri.h"
 #endif
 
-#include "mach64_common.h"
-
-
-
-#include "xf86.h"
-
 #ifdef TV_OUT
 
 #include "atichip.h"
@@ -93,23 +84,8 @@ ATISaveScreen
         return TRUE;
 
     pATI = ATIPTR(pScreenInfo);
-    switch (pATI->NewHW.crtc)
     {
-
-#ifndef AVOID_CPIO
-
-        case ATI_CRTC_VGA:
-            ATIVGASaveScreen(pATI, Mode);
-            break;
-
-#endif /* AVOID_CPIO */
-
-        case ATI_CRTC_MACH64:
             ATIMach64SaveScreen(pATI, Mode);
-            break;
-
-        default:
-            break;
     }
 
     return TRUE;
@@ -136,27 +112,8 @@ ATISetDPMSMode
 
     pATI = ATIPTR(pScreenInfo);
 
-    switch (pATI->Adapter)
     {
-        case ATI_ADAPTER_MACH64:
             ATIMach64SetDPMSMode(pScreenInfo, pATI, DPMSMode);
-            break;
-
-        default:
-
-#ifndef AVOID_CPIO
-
-            /* Assume EGA/VGA */
-            ATIVGASetDPMSMode(pATI, DPMSMode);
-            break;
-
-        case ATI_ADAPTER_NONE:
-        case ATI_ADAPTER_8514A:
-        case ATI_ADAPTER_MACH8:
-
-#endif /* AVOID_CPIO */
-
-            break;
     }
 }
 
@@ -671,7 +628,7 @@ ATILeaveGraphics
  *
  * This function switches to another graphics video state.
  */
-_X_EXPORT Bool
+Bool
 ATISwitchMode
 (
     int            iScreen,
@@ -724,7 +681,7 @@ ATISwitchMode
  *
  * This function sets the server's virtual console to a graphics video state.
  */
-_X_EXPORT Bool
+Bool
 ATIEnterVT
 (
     int iScreen,
@@ -759,14 +716,6 @@ ATIEnterVT
         return TRUE;
     }
 
-#ifndef AVOID_CPIO
-
-    /* If used, modify banking interface */
-    if (!miModifyBanking(pScreen, &pATI->BankInfo))
-        return FALSE;
-
-#endif /* AVOID_CPIO */
-
     pScreenPixmap = (*pScreen->GetScreenPixmap)(pScreen);
     PixmapPrivate = pScreenPixmap->devPrivate;
     if (!PixmapPrivate.ptr)
@@ -802,7 +751,7 @@ ATIEnterVT
  * This function restores the server's virtual console to its state on server
  * entry.
  */
-_X_EXPORT void
+void
 ATILeaveVT
 (
     int iScreen,
@@ -831,7 +780,7 @@ ATILeaveVT
  *
  * This function frees all driver data related to a screen.
  */
-_X_EXPORT void
+void
 ATIFreeScreen
 (
     int iScreen,
