@@ -1,4 +1,3 @@
-/* $XFree86$ */
 /*
  * Copyright 2003 Alex Deucher.
  *
@@ -36,6 +35,9 @@
  *   Based, in large part, on the sis driver by Thomas Winischhofer.
  */
 
+#include <string.h>
+#include <stdio.h>
+
 #include "xf86.h"
 #include "xf86Priv.h"
 #include "xf86Resources.h"
@@ -209,12 +211,19 @@ RADEONCopyModeNLink(ScrnInfoPtr pScrn, DisplayModePtr dest,
     mode->VSyncEnd += dy;
     mode->VTotal += dy;
 
-     /* Provide a sophisticated fake DotClock in order to trick the vidmode
+     /* Provide a fake VRefresh/DotClock in order to trick the vidmode
       * extension to allow selecting among a number of modes whose merged result
       * looks identical but consists of different modes for CRT1 and CRT2
       */
-    mode->Clock = (((i->Clock >> 3) + i->HTotal) << 16) | ((j->Clock >> 2) + j->HTotal);
-    mode->Clock ^= ((i->VTotal << 19) | (j->VTotal << 3));
+    {
+	float ref1, ref2;
+	ref1 = ((float)i->Clock * 100.0 / i->HTotal / i->VTotal) * 50.0;
+	ref2 = ((float)j->Clock * 100.0 / j->HTotal / j->VTotal) / 2.0;
+
+        mode->VRefresh = (float) ref1 + ref2;
+    }
+
+    mode->Clock = (int)(mode->VRefresh * 0.001 * mode->HTotal * mode->VTotal);
 
     if( ((mode->HDisplay * ((pScrn->bitsPerPixel + 7) / 8) * mode->VDisplay) > 
 	(pScrn->videoRam * 1024)) ||
@@ -344,8 +353,6 @@ RADEONGenerateModeListFromLargestModes(ScrnInfoPtr pScrn,
     DisplayModePtr mode3 = NULL;
     DisplayModePtr mode4 = NULL;
     DisplayModePtr result = NULL;
-    int p = 0;
-    int count = 0;
 
     info->AtLeastOneNonClone = FALSE;
 
@@ -387,7 +394,7 @@ RADEONGenerateModeListFromLargestModes(ScrnInfoPtr pScrn,
     if(srel != radeonClone) {
        if(mode3 && mode4 && !info->NonRect) {
 	  mode1 = mode3;
-	  mode2 = mode2;
+	  mode2 = mode4;
        }
     }
 
@@ -413,7 +420,8 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
 {
     char* strmode = str;
     char modename[256];
-    Bool gotdash = FALSE, gotplus = FALSE;
+    Bool gotdash = FALSE;
+    char gotsep = 0;
     RADEONScrn2Rel sr;
     DisplayModePtr mode1 = NULL;
     DisplayModePtr mode2 = NULL;
@@ -429,6 +437,8 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
         case '-':
 	case '+':
         case ' ':
+	case ',':
+	case ';':
            if(strmode != str) {
 
               myslen = str - strmode;
@@ -447,9 +457,9 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
                     xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                         "Mode \"%s\" is not a supported mode for CRT2\n", modename);
                     xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
-                        "\t(Skipping metamode \"%s%s%s\")\n", mode1->name, gotplus ? "+" : "-", modename);
+                        "\t(Skipping metamode \"%s%c%s\")\n", mode1->name, gotsep, modename);
                     mode1 = NULL;
-		    gotplus = FALSE;
+		    gotsep = 0;
                  }
               } else {
                  mode1 = RADEONGetModeFromName(modename, i);
@@ -457,14 +467,14 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
                     char* tmps = str;
                     xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                         "Mode \"%s\" is not a supported mode for CRT1\n", modename);
-                    while(*tmps == ' ') tmps++;
+                    while(*tmps == ' ' || *tmps == ';') tmps++;
                     /* skip the next mode */
-  	            if((*tmps == '-') || (*tmps == '+')) {
+  	            if(*tmps == '-' || *tmps == '+' || *tmps == ',') {
                        tmps++;
 		       /* skip spaces */
-		       while(*tmps == ' ') tmps++;
+		       while(*tmps == ' ' || *tmps == ';') tmps++;
 		       /* skip modename */
-		       while((*tmps != ' ') && (*tmps != '-') && (*tmps != '+') && (*tmps != 0)) tmps++;
+		       while(*tmps && *tmps != ' ' && *tmps != ';' && *tmps != '-' && *tmps != '+' && *tmps != ',') tmps++;
   	               myslen = tmps - strmode;
   	               if(myslen > 255) myslen = 255;
   	               strncpy(modename,strmode,myslen);
@@ -474,14 +484,15 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
                     xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
                         "\t(Skipping metamode \"%s\")\n", modename);
                     mode1 = NULL;
-		    gotplus = FALSE;
+		    gotsep = 0;
                  }
               }
               gotdash = FALSE;
            }
            strmode = str + 1;
-           gotdash |= ((*str == '-') || (*str == '+'));
-  	   gotplus |= (*str == '+');
+           gotdash |= (*str == '-' || *str == '+' || *str == ',');
+	   if (*str == '-' || *str == '+' || *str == ',')
+  	      gotsep = *str;
 
            if(*str != 0) break;
 	   /* Fall through otherwise */
@@ -489,7 +500,7 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
         default:
            if(!gotdash && mode1) {
               sr = srel;
-	      if(gotplus) sr = radeonClone;
+	      if(gotsep == '+') sr = radeonClone;
               if(!mode2) {
                  mode2 = RADEONGetModeFromName(mode1->name, j);
                  sr = radeonClone;
@@ -505,7 +516,7 @@ RADEONGenerateModeListFromMetaModes(ScrnInfoPtr pScrn, char* str,
                  mode1 = NULL;
                  mode2 = NULL;
               }
-	      gotplus = FALSE;
+	      gotsep = 0;
            }
            break;
 
@@ -526,15 +537,48 @@ RADEONGenerateModeList(ScrnInfoPtr pScrn, char* str,
    if(str != NULL) {
       return(RADEONGenerateModeListFromMetaModes(pScrn, str, i, j, srel));
    } else {
-      xf86DrvMsg(pScrn->scrnIndex, X_INFO,
-	"No MetaModes given, linking %s modes by default\n",
-	(srel == radeonClone) ? "largest common" :
-	   (info->NonRect ?
+        if (srel == radeonClone ) {
+           DisplayModePtr p, q, result = NULL;
+
+           xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                      "Clone mode, linking all nearest modes\n");
+
+           p = i;
+           q = j;
+
+           result = RADEONCopyModeNLink(pScrn, result, p, q, srel);
+
+           while (p->next != i || q->next != j) {
+              DisplayModePtr next_p = p;
+
+              if (q->next == j || (p->next != i &&
+                                   (p->HDisplay > q->HDisplay ||
+                                    (p->HDisplay == q->HDisplay &&
+                                     p->VDisplay >= q->VDisplay))))
+                 next_p = p->next;
+
+              if (p->next == i || (q->next != j &&
+                                   (q->HDisplay > p->HDisplay ||
+                                    (q->HDisplay == p->HDisplay &&
+                                     q->VDisplay >= p->VDisplay))))
+                 q = q->next;
+
+              p = next_p;
+
+              result = RADEONCopyModeNLink(pScrn, result, p, q, srel);
+           }
+
+           return result;
+        } else {
+           xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                "No MetaModes given, linking %s modes by default\n",
+                (info->NonRect ?
 		(((srel == radeonLeftOf) || (srel == radeonRightOf)) ? "widest" :  "tallest")
 		:
 		(((srel == radeonLeftOf) || (srel == radeonRightOf)) ? "widest common" :  "tallest common")) );
-      return(RADEONGenerateModeListFromLargestModes(pScrn, i, j, srel));
-   }
+           return(RADEONGenerateModeListFromLargestModes(pScrn, i, j, srel));
+        }
+    }
 }
 
 void
@@ -563,8 +607,10 @@ RADEONRecalcDefaultVirtualSize(ScrnInfoPtr pScrn)
   	               info->CRT1XOffs = info->CRT2XOffs = 0;
   	               maxh -= (info->CRT1XOffs + info->CRT2XOffs);
   	}
-  	pScrn->virtualX = maxh;
-  	pScrn->displayWidth = maxh;
+	if (maxh > pScrn->virtualX)
+	    pScrn->virtualX = maxh;
+	if (maxh > pScrn->displayWidth)
+	    pScrn->displayWidth = maxh;
   	xf86DrvMsg(pScrn->scrnIndex, X_PROBED, str, "width", maxh);
     } else {
   	if(maxh < pScrn->display->virtualX) {
@@ -574,7 +620,8 @@ RADEONRecalcDefaultVirtualSize(ScrnInfoPtr pScrn)
     }
 
     if(!(pScrn->display->virtualY)) {
-        pScrn->virtualY = maxv;
+	if (maxv > pScrn->virtualY)
+	    pScrn->virtualY = maxv;
 	xf86DrvMsg(pScrn->scrnIndex, X_PROBED, str, "height", maxv);
     } else {
 	if(maxv < pScrn->display->virtualY) {
@@ -1734,119 +1781,125 @@ RADEONAdjustFrameMerged(int scrnIndex, int x, int y, int flags)
     RADEONDoAdjustFrame(pScrn1, pScrn2->frameX0, pScrn2->frameY0, TRUE);
 }
 
-void
-RADEONMergedFBSetDpi(ScrnInfoPtr pScrn1, ScrnInfoPtr pScrn2, RADEONScrn2Rel srel)
+static void
+RADEONMergedFBCalcDPI(ScrnInfoPtr pScrn1, ScrnInfoPtr pScrn2, RADEONScrn2Rel srel, Bool quiet)
 {
-   RADEONInfoPtr      info       = RADEONPTR(pScrn1);
-   MessageType from = X_DEFAULT;
-   xf86MonPtr DDC1 = (xf86MonPtr)(pScrn1->monitor->DDC);
-   xf86MonPtr DDC2 = (xf86MonPtr)(pScrn2->monitor->DDC);
-   int ddcWidthmm = 0, ddcHeightmm = 0;
-   const char *dsstr = "MergedFB: Display dimensions: (%d, %d) mm\n";
+    RADEONInfoPtr info = RADEONPTR(pScrn1);
+    MessageType from = X_DEFAULT;
+    xf86MonPtr DDC1 = (xf86MonPtr)(pScrn1->monitor->DDC);
+    xf86MonPtr DDC2 = (xf86MonPtr)(pScrn2->monitor->DDC);
+    int ddcWidthmm = 0, ddcHeightmm = 0;
+    const char *dsstr = "MergedFB: Display dimensions: %dx%d mm\n";
 
-   /* This sets the DPI for MergedFB mode. The problem is that
-    * this can never be exact, because the output devices may
-    * have different dimensions. This function tries to compromise
-    * through a few assumptions, and it just calculates an average DPI
-    * value for both monitors.
-    */
+    /* This sets the DPI for MergedFB mode. The problem is that
+     * this can never be exact, because the output devices may
+     * have different dimensions. This function tries to compromise
+     * through a few assumptions, and it just calculates an average
+     * DPI value for both monitors.
+     */
 
-   /* Given DisplaySize should regard BOTH monitors */
-   pScrn1->widthmm = pScrn1->monitor->widthmm;
-   pScrn1->heightmm = pScrn1->monitor->heightmm;
+    /* Copy user-given DisplaySize (which should regard BOTH monitors!) */
+    pScrn1->widthmm = pScrn1->monitor->widthmm;
+    pScrn1->heightmm = pScrn1->monitor->heightmm;
 
-   /* Get DDC display size; if only either CRT1 or CRT2 provided these,
-    * assume equal dimensions for both, otherwise add dimensions
-    */
-   if( (DDC1 && (DDC1->features.hsize > 0 && DDC1->features.vsize > 0)) &&
-       (DDC2 && (DDC2->features.hsize > 0 && DDC2->features.vsize > 0)) ) {
-      ddcWidthmm = max(DDC1->features.hsize, DDC2->features.hsize) * 10;
-      ddcHeightmm = max(DDC1->features.vsize, DDC2->features.vsize) * 10;
-      switch(srel) {
-      case radeonLeftOf:
-      case radeonRightOf:
-         ddcWidthmm = (DDC1->features.hsize + DDC2->features.hsize) * 10;
-	 break;
-      case radeonAbove:
-      case radeonBelow:
-         ddcHeightmm = (DDC1->features.vsize + DDC2->features.vsize) * 10;
-      default:
-	 break;
-      }
+    if(monitorResolution > 0) {
 
-   } else if(DDC1 && (DDC1->features.hsize > 0 && DDC1->features.vsize > 0)) {
-      ddcWidthmm = DDC1->features.hsize * 10;
-      ddcHeightmm = DDC1->features.vsize * 10;
-      switch(srel) {
-      case radeonLeftOf:
-      case radeonRightOf:
-         ddcWidthmm *= 2;
-	 break;
-      case radeonAbove:
-      case radeonBelow:
-         ddcHeightmm *= 2;
-      default:
-	 break;
-      }
-   } else if(DDC2 && (DDC2->features.hsize > 0 && DDC2->features.vsize > 0) ) {
-      ddcWidthmm = DDC2->features.hsize * 10;
-      ddcHeightmm = DDC2->features.vsize * 10;
-      switch(srel) {
-      case radeonLeftOf:
-      case radeonRightOf:
-         ddcWidthmm *= 2;
-	 break;
-      case radeonAbove:
-      case radeonBelow:
-         ddcHeightmm *= 2;
-      default:
-	 break;
-      }
-   }
+       /* Set command line given values (overrules given options) */
+       pScrn1->xDpi = monitorResolution;
+       pScrn1->yDpi = monitorResolution;
+       from = X_CMDLINE;
 
-   if(monitorResolution > 0) {
+    } else if(info->MergedFBXDPI) {
 
-      /* Set command line given values (overrules given options) */
-      pScrn1->xDpi = monitorResolution;
-      pScrn1->yDpi = monitorResolution;
-      from = X_CMDLINE;
+       /* Set option-wise given values (overrules DisplaySize config option) */
+       pScrn1->xDpi = info->MergedFBXDPI;
+       pScrn1->yDpi = info->MergedFBYDPI;
+       from = X_CONFIG;
 
-   } else if(info->MergedFBXDPI) {
+    } else if(pScrn1->widthmm > 0 || pScrn1->heightmm > 0) {
 
-      /* Set option-wise given values (overrule DisplaySize) */
-      pScrn1->xDpi = info->MergedFBXDPI;
-      pScrn1->yDpi = info->MergedFBYDPI;
-      from = X_CONFIG;
-
-   } else if(pScrn1->widthmm > 0 || pScrn1->heightmm > 0) {
-
-      /* Set values calculated from given DisplaySize */
-      from = X_CONFIG;
-      if(pScrn1->widthmm > 0) {
-	 pScrn1->xDpi = (int)((double)pScrn1->virtualX * 25.4 / pScrn1->widthmm);
-      }
-      if(pScrn1->heightmm > 0) {
-	 pScrn1->yDpi = (int)((double)pScrn1->virtualY * 25.4 / pScrn1->heightmm);
-      }
-      xf86DrvMsg(pScrn1->scrnIndex, from, dsstr, pScrn1->widthmm, pScrn1->heightmm);
+       /* Set values calculated from given DisplaySize */
+       from = X_CONFIG;
+       if(pScrn1->widthmm > 0) {
+	  pScrn1->xDpi = (int)((double)pScrn1->virtualX * 25.4 / pScrn1->widthmm);
+       }
+       if(pScrn1->heightmm > 0) {
+	  pScrn1->yDpi = (int)((double)pScrn1->virtualY * 25.4 / pScrn1->heightmm);
+       }
+       if(!quiet) {
+          xf86DrvMsg(pScrn1->scrnIndex, from, dsstr, pScrn1->widthmm, pScrn1->heightmm);
+       }
 
     } else if(ddcWidthmm && ddcHeightmm) {
 
-      /* Set values from DDC-provided display size */
-      from = X_PROBED;
-      xf86DrvMsg(pScrn1->scrnIndex, from, dsstr, ddcWidthmm, ddcHeightmm );
-      pScrn1->widthmm = ddcWidthmm;
-      pScrn1->heightmm = ddcHeightmm;
-      if(pScrn1->widthmm > 0) {
-	 pScrn1->xDpi = (int)((double)pScrn1->virtualX * 25.4 / pScrn1->widthmm);
-      }
-      if(pScrn1->heightmm > 0) {
-	 pScrn1->yDpi = (int)((double)pScrn1->virtualY * 25.4 / pScrn1->heightmm);
-      }
+       /* Set values from DDC-provided display size */
+
+       /* Get DDC display size; if only either CRT1 or CRT2 provided these,
+	* assume equal dimensions for both, otherwise add dimensions
+	*/
+       if( (DDC1 && (DDC1->features.hsize > 0 && DDC1->features.vsize > 0)) &&
+	   (DDC2 && (DDC2->features.hsize > 0 && DDC2->features.vsize > 0)) ) {
+	  ddcWidthmm = max(DDC1->features.hsize, DDC2->features.hsize) * 10;
+	  ddcHeightmm = max(DDC1->features.vsize, DDC2->features.vsize) * 10;
+	  switch(srel) {
+	  case radeonLeftOf:
+	  case radeonRightOf:
+	     ddcWidthmm = (DDC1->features.hsize + DDC2->features.hsize) * 10;
+	     break;
+	  case radeonAbove:
+	  case radeonBelow:
+	     ddcHeightmm = (DDC1->features.vsize + DDC2->features.vsize) * 10;
+	  default:
+	     break;
+	  }
+       } else if(DDC1 && (DDC1->features.hsize > 0 && DDC1->features.vsize > 0)) {
+	  ddcWidthmm = DDC1->features.hsize * 10;
+	  ddcHeightmm = DDC1->features.vsize * 10;
+	  switch(srel) {
+	  case radeonLeftOf:
+	  case radeonRightOf:
+	     ddcWidthmm *= 2;
+	     break;
+	  case radeonAbove:
+	  case radeonBelow:
+	     ddcHeightmm *= 2;
+	  default:
+	     break;
+          }
+       } else if(DDC2 && (DDC2->features.hsize > 0 && DDC2->features.vsize > 0) ) {
+	  ddcWidthmm = DDC2->features.hsize * 10;
+	  ddcHeightmm = DDC2->features.vsize * 10;
+	  switch(srel) {
+	  case radeonLeftOf:
+	  case radeonRightOf:
+	     ddcWidthmm *= 2;
+	     break;
+	  case radeonAbove:
+	  case radeonBelow:
+	     ddcHeightmm *= 2;
+	  default:
+	     break;
+	  }
+       }
+
+       from = X_PROBED;
+
+       if(!quiet) {
+          xf86DrvMsg(pScrn1->scrnIndex, from, dsstr, ddcWidthmm, ddcHeightmm);
+       }
+
+       pScrn1->widthmm = ddcWidthmm;
+       pScrn1->heightmm = ddcHeightmm;
+       if(pScrn1->widthmm > 0) {
+	  pScrn1->xDpi = (int)((double)pScrn1->virtualX * 25.4 / pScrn1->widthmm);
+       }
+       if(pScrn1->heightmm > 0) {
+	  pScrn1->yDpi = (int)((double)pScrn1->virtualY * 25.4 / pScrn1->heightmm);
+       }
 
     } else {
 
-      pScrn1->xDpi = pScrn1->yDpi = DEFAULT_DPI;
+       pScrn1->xDpi = pScrn1->yDpi = DEFAULT_DPI;
 
     }
 
@@ -1859,8 +1912,60 @@ RADEONMergedFBSetDpi(ScrnInfoPtr pScrn1, ScrnInfoPtr pScrn2, RADEONScrn2Rel srel
     pScrn2->xDpi = pScrn1->xDpi;
     pScrn2->yDpi = pScrn1->yDpi;
 
-    xf86DrvMsg(pScrn1->scrnIndex, from, "MergedFB: DPI set to (%d, %d)\n",
-	       pScrn1->xDpi, pScrn1->yDpi);
+    if(!quiet) {
+       xf86DrvMsg(pScrn1->scrnIndex, from, "MergedFB: DPI set to (%d, %d)\n",
+		pScrn1->xDpi, pScrn1->yDpi);
+    }
+}
+
+
+void
+RADEONMergedFBSetDpi(ScrnInfoPtr pScrn1, ScrnInfoPtr pScrn2, RADEONScrn2Rel srel)
+{
+    RADEONInfoPtr      info       = RADEONPTR(pScrn1);
+
+    RADEONMergedFBCalcDPI(pScrn1, pScrn2, srel, FALSE);
+
+    info->MergedDPISRel = srel;
+    info->RADEONMergedDPIVX = pScrn1->virtualX;
+    info->RADEONMergedDPIVY = pScrn1->virtualY;
+
+}
+
+void
+RADEONMergedFBResetDpi(ScrnInfoPtr pScrn, Bool force)
+{
+    RADEONInfoPtr info = RADEONPTR(pScrn);
+    ScreenPtr pScreen = screenInfo.screens[pScrn->scrnIndex];
+    RADEONScrn2Rel srel = ((RADEONMergedDisplayModePtr)info->CurrentLayout.mode->Private)->CRT2Position;
+
+    /* This does the same calculation for the DPI as
+     * the initial run. This means that an eventually
+     * given -dpi command line switch will lead to
+     * constant dpi values, regardless of the virtual
+     * screen size.
+     * I consider this consequent. If this is undesired,
+     * one should use the DisplaySize parameter in the
+     * config file instead of the command line switch.
+     * The DPI will be calculated then.
+     */
+
+    if(force						||
+       (info->MergedDPISRel != srel)			||
+       (info->RADEONMergedDPIVX != pScrn->virtualX)	||
+       (info->RADEONMergedDPIVY != pScrn->virtualY)
+						) {
+
+       RADEONMergedFBCalcDPI(pScrn, info->CRT2pScrn, srel, TRUE);
+
+       pScreen->mmWidth = (pScrn->virtualX * 254 + pScrn->xDpi * 5) / (pScrn->xDpi * 10);
+       pScreen->mmHeight = (pScrn->virtualY * 254 + pScrn->yDpi * 5) / (pScrn->yDpi * 10);
+
+       info->MergedDPISRel = srel;
+       info->RADEONMergedDPIVX = pScrn->virtualX;
+       info->RADEONMergedDPIVY = pScrn->virtualY;
+
+    }
 }
 
 /* radeon cursor helpers */
@@ -1961,14 +2066,7 @@ RADEONSetCursorPositionMerged(ScrnInfoPtr pScrn, int x, int y)
     OUTREG(RADEON_CUR_HORZ_VERT_POSN, (RADEON_CUR_LOCK
 				   | ((xorigin ? 0 : x1) << 16)
 				   | (yorigin ? 0 : y1)));
-#ifdef USE_EXA
-    if (info->useEXA)
-	OUTREG(RADEON_CUR_OFFSET, info->cursorArea->offset + yorigin * stride);
-#endif /* USE_EXA */
-#ifdef USE_XAA
-    if (!info->useEXA)
-	OUTREG(RADEON_CUR_OFFSET, info->cursor_offset + yorigin * stride);
-#endif /* USE_XAA */
+    OUTREG(RADEON_CUR_OFFSET, info->cursor_offset + yorigin * stride);
 		/* cursor2 */
     OUTREG(RADEON_CUR2_HORZ_VERT_OFF,  (RADEON_CUR2_LOCK
 				    | (xorigin << 16)
@@ -1976,14 +2074,7 @@ RADEONSetCursorPositionMerged(ScrnInfoPtr pScrn, int x, int y)
     OUTREG(RADEON_CUR2_HORZ_VERT_POSN, (RADEON_CUR2_LOCK
 				    | ((xorigin ? 0 : x2) << 16)
 				    | (yorigin ? 0 : y2)));
-#ifdef USE_EXA
-    if (info->useEXA)
-	OUTREG(RADEON_CUR2_OFFSET, info->cursorArea->offset + yorigin * stride);
-#endif /* USE_EXA */
-#ifdef USE_XAA
-    if (!info->useEXA)
-	OUTREG(RADEON_CUR2_OFFSET, info->cursor_offset + yorigin * stride);
-#endif /* USE_XAA */
+    OUTREG(RADEON_CUR2_OFFSET, info->cursor_offset + yorigin * stride);
 }
 
 /* radeon Xv helpers */
